import pytest
from lxml import etree

from verbecc.src.defs.types.gender import Gender
from verbecc.src.defs.types.person import Person
from verbecc.src.defs.types.lang_code import LangCodeISO639_1 as Lang
from verbecc.src.defs.types.mood import Mood
from verbecc.src.defs.types.tense import Tense
from verbecc.src.defs.types.lang_specific_options import LangSpecificOptions
from verbecc.src.defs.types.lang.es.lang_specific_options_es import (
    LangSpecificOptionsEs,
)
from verbecc.src.defs.types.lang.es.voseo_options import VoseoOptions
from verbecc.src.conjugator.conjugator import Conjugator
from verbecc.src.parsers.tense_template import TenseTemplate

cg = Conjugator(lang="es")


def test_all_verbs_have_templates():
    verbs = cg.get_verbs()
    template_names = cg.get_template_names()
    missing_templates = set()
    for verb in verbs:
        if verb.template not in template_names:
            missing_templates.add(verb.template)
    assert len(missing_templates) == 0


# presente = Subjunctive Present (yo haya)
# pretérito-perfecto = Subjunctive Perfect (yo haya habido)
# pretérito-imperfecto-1 = Subjunctive Past 1 (yo hubiera)
# pretérito-imperfecto-2 = Subjunctive Past 2 (yo hubiese)
# pretérito-pluscuamperfecto-1 = Subjunctive Pluperfect 1 (yo hubiera habido)
# pretérito-pluscuamperfecto-2 = Subjunctive Pluperfect 2 (yo hubiese habido)
# futuro = Subjunctive Future (yo hubiere)
# futuro-perfecto = Subjunctive Future Perfect (yo hubiere habido)
@pytest.mark.parametrize(
    "infinitive,mood,tense,expected_result",
    [
        (
            "abañar",
            "indicativo",
            "presente",
            [
                "yo abaño",
                "tú abañas",
                "él abaña",
                "nosotros abañamos",
                "vosotros abañáis",
                "ellos abañan",
            ],
        ),
        (
            "estar",
            "indicativo",
            "presente",
            [
                "yo estoy",
                "tú estás",
                "él está",
                "nosotros estamos",
                "vosotros estáis",
                "ellos están",
            ],
        ),
        (
            "ser",
            "indicativo",
            "presente",
            [
                "yo soy",
                "tú eres",
                "él es",
                "nosotros somos",
                "vosotros sois",
                "ellos son",
            ],
        ),
        (
            "tener",
            "indicativo",
            "presente",
            [
                "yo tengo",
                "tú tienes",
                "él tiene",
                "nosotros tenemos",
                "vosotros tenéis",
                "ellos tienen",
            ],
        ),
        (
            "haber",
            "indicativo",
            "presente",
            [
                "yo he",
                "tú has",
                "él hay",
                "nosotros hemos",
                "vosotros habéis",
                "ellos han",
            ],
        ),
        (
            "haber",
            "indicativo",
            "pretérito-imperfecto",
            [
                "yo había",
                "tú habías",
                "él había",
                "nosotros habíamos",
                "vosotros habíais",
                "ellos habían",
            ],
        ),
        (
            "haber",
            "indicativo",
            "pretérito-perfecto-simple",
            [
                "yo hube",
                "tú hubiste",
                "él hubo",
                "nosotros hubimos",
                "vosotros hubisteis",
                "ellos hubieron",
            ],
        ),
        (
            "haber",
            "indicativo",
            "futuro",
            [
                "yo habré",
                "tú habrás",
                "él habrá",
                "nosotros habremos",
                "vosotros habréis",
                "ellos habrán",
            ],
        ),
        (
            "haber",
            "condicional",
            "presente",
            [
                "yo habría",
                "tú habrías",
                "él habría",
                "nosotros habríamos",
                "vosotros habríais",
                "ellos habrían",
            ],
        ),
        (
            "haber",
            "subjuntivo",
            "presente",
            [
                "yo haya",
                "tú hayas",
                "él haya",
                "nosotros hayamos",
                "vosotros hayáis",
                "ellos hayan",
            ],
        ),
        (
            "haber",
            "subjuntivo",
            "pretérito-imperfecto-1",
            [
                "yo hubiera",
                "tú hubieras",
                "él hubiera",
                "nosotros hubiéramos",
                "vosotros hubierais",
                "ellos hubieran",
            ],
        ),
        (
            "haber",
            "subjuntivo",
            "pretérito-imperfecto-2",
            [
                "yo hubiese",
                "tú hubieses",
                "él hubiese",
                "nosotros hubiésemos",
                "vosotros hubieseis",
                "ellos hubiesen",
            ],
        ),
        (
            "haber",
            "subjuntivo",
            "futuro",
            [
                "yo hubiere",
                "tú hubieres",
                "él hubiere",
                "nosotros hubiéremos",
                "vosotros hubiereis",
                "ellos hubieren",
            ],
        ),
        (
            "hacer",
            "indicativo",
            "presente",
            [
                "yo hago",
                "tú haces",
                "él hace",
                "nosotros hacemos",
                "vosotros hacéis",
                "ellos hacen",
            ],
        ),
        (
            "ir",
            "indicativo",
            "presente",
            [
                "yo voy",
                "tú vas",
                "él va",
                "nosotros vamos",
                "vosotros vais",
                "ellos van",
            ],
        ),
        (
            "comer",
            "indicativo",
            "presente",
            [
                "yo como",
                "tú comes",
                "él come",
                "nosotros comemos",
                "vosotros coméis",
                "ellos comen",
            ],
        ),
        (
            "comer",
            "indicativo",
            "pretérito-perfecto-simple",
            [
                "yo comí",
                "tú comiste",
                "él comió",
                "nosotros comimos",
                "vosotros comisteis",
                "ellos comieron",
            ],
        ),
        (
            "comer",
            "indicativo",
            "pretérito-imperfecto",
            [
                "yo comía",
                "tú comías",
                "él comía",
                "nosotros comíamos",
                "vosotros comíais",
                "ellos comían",
            ],
        ),
        (
            "comer",
            "condicional",
            "presente",
            [
                "yo comería",
                "tú comerías",
                "él comería",
                "nosotros comeríamos",
                "vosotros comeríais",
                "ellos comerían",
            ],
        ),
        (
            "comer",
            "indicativo",
            "pretérito-perfecto-compuesto",
            [
                "yo he comido",
                "tú has comido",
                "él ha comido",
                "nosotros hemos comido",
                "vosotros habéis comido",
                "ellos han comido",
            ],
        ),
        (
            "comer",
            "indicativo",
            "pretérito-pluscuamperfecto",
            [
                "yo había comido",
                "tú habías comido",
                "él había comido",
                "nosotros habíamos comido",
                "vosotros habíais comido",
                "ellos habían comido",
            ],
        ),
        (
            "comer",
            "indicativo",
            "pretérito-anterior",
            [
                "yo hube comido",
                "tú hubiste comido",
                "él hubo comido",
                "nosotros hubimos comido",
                "vosotros hubisteis comido",
                "ellos hubieron comido",
            ],
        ),
        (
            "comer",
            "indicativo",
            "futuro-perfecto",
            [
                "yo habré comido",
                "tú habrás comido",
                "él habrá comido",
                "nosotros habremos comido",
                "vosotros habréis comido",
                "ellos habrán comido",
            ],
        ),
        (
            "comer",
            "condicional",
            "perfecto",
            [
                "yo habría comido",
                "tú habrías comido",
                "él habría comido",
                "nosotros habríamos comido",
                "vosotros habríais comido",
                "ellos habrían comido",
            ],
        ),
        (
            "comer",
            "subjuntivo",
            "pretérito-perfecto",
            [
                "yo haya comido",
                "tú hayas comido",
                "él haya comido",
                "nosotros hayamos comido",
                "vosotros hayáis comido",
                "ellos hayan comido",
            ],
        ),
        (
            "comer",
            "subjuntivo",
            "pretérito-pluscuamperfecto-1",
            [
                "yo hubiera comido",
                "tú hubieras comido",
                "él hubiera comido",
                "nosotros hubiéramos comido",
                "vosotros hubierais comido",
                "ellos hubieran comido",
            ],
        ),
        (
            "comer",
            "subjuntivo",
            "pretérito-pluscuamperfecto-2",
            [
                "yo hubiese comido",
                "tú hubieses comido",
                "él hubiese comido",
                "nosotros hubiésemos comido",
                "vosotros hubieseis comido",
                "ellos hubiesen comido",
            ],
        ),
        (
            "comer",
            "subjuntivo",
            "futuro-perfecto",
            [
                "yo hubiere comido",
                "tú hubieres comido",
                "él hubiere comido",
                "nosotros hubiéremos comido",
                "vosotros hubiereis comido",
                "ellos hubieren comido",
            ],
        ),
        (
            "comer",
            "imperativo",
            "afirmativo",
            ["come", "coma", "comamos", "comed", "coman"],
        ),
        (
            "comer",
            "imperativo",
            "negativo",
            ["no comas", "no coma", "no comamos", "no comáis", "no coman"],
        ),
        (
            "parecer",
            "indicativo",
            "presente",
            [
                "yo parezco",
                "tú pareces",
                "él parece",
                "nosotros parecemos",
                "vosotros parecéis",
                "ellos parecen",
            ],
        ),
        (
            "parecer",
            "indicativo",
            "pretérito-imperfecto",
            [
                "yo parecía",
                "tú parecías",
                "él parecía",
                "nosotros parecíamos",
                "vosotros parecíais",
                "ellos parecían",
            ],
        ),
        (
            "parecer",
            "indicativo",
            "pretérito-perfecto-simple",
            [
                "yo parecí",
                "tú pareciste",
                "él pareció",
                "nosotros parecimos",
                "vosotros parecisteis",
                "ellos parecieron",
            ],
        ),
        (
            "parecer",
            "indicativo",
            "futuro",
            [
                "yo pareceré",
                "tú parecerás",
                "él parecerá",
                "nosotros pareceremos",
                "vosotros pareceréis",
                "ellos parecerán",
            ],
        ),
        (
            "parecer",
            "subjuntivo",
            "presente",
            [
                "yo parezca",
                "tú parezcas",
                "él parezca",
                "nosotros parezcamos",
                "vosotros parezcáis",
                "ellos parezcan",
            ],
        ),
        (
            "parecer",
            "subjuntivo",
            "pretérito-imperfecto-1",
            [
                "yo pareciera",
                "tú parecieras",
                "él pareciera",
                "nosotros pareciéramos",
                "vosotros parecierais",
                "ellos parecieran",
            ],
        ),
        (
            "parecer",
            "subjuntivo",
            "pretérito-imperfecto-2",
            [
                "yo pareciese",
                "tú parecieses",
                "él pareciese",
                "nosotros pareciésemos",
                "vosotros parecieseis",
                "ellos pareciesen",
            ],
        ),
        (
            "parecer",
            "subjuntivo",
            "futuro",
            [
                "yo pareciere",
                "tú parecieres",
                "él pareciere",
                "nosotros pareciéremos",
                "vosotros pareciereis",
                "ellos parecieren",
            ],
        ),
        (
            "parecer",
            "imperativo",
            "afirmativo",
            ["parece", "parezca", "parezcamos", "pareced", "parezcan"],
        ),
        (
            "parecer",
            "imperativo",
            "negativo",
            [
                "no parezcas",
                "no parezca",
                "no parezcamos",
                "no parezcáis",
                "no parezcan",
            ],
        ),
        (
            "parecer",
            "condicional",
            "presente",
            [
                "yo parecería",
                "tú parecerías",
                "él parecería",
                "nosotros pareceríamos",
                "vosotros pareceríais",
                "ellos parecerían",
            ],
        ),
        (
            "parecer",
            "indicativo",
            "pretérito-perfecto-compuesto",
            [
                "yo he parecido",
                "tú has parecido",
                "él ha parecido",
                "nosotros hemos parecido",
                "vosotros habéis parecido",
                "ellos han parecido",
            ],
        ),
        (
            "abolir",
            "indicativo",
            "presente",
            [
                "yo abolo",
                "tú aboles",
                "él abole",
                "nosotros abolimos",
                "vosotros abolís",
                "ellos abolen",
            ],
        ),
        (
            "abolir",
            "subjuntivo",
            "futuro",
            [
                "yo aboliere",
                "tú abolieres",
                "él aboliere",
                "nosotros aboliéremos",
                "vosotros aboliereis",
                "ellos abolieren",
            ],
        ),
    ],
)
def test_inflector_es_conjugate_mood_tense(infinitive, mood, tense, expected_result):
    assert cg.conjugate_mood_tense(infinitive, mood, tense) == expected_result


def test_abolir():
    """
    Reproduce error:

    >           co.template.mood_templates[persons_mood].tense_templates[aux_tense].person_endings]
    E       KeyError: 'presente'

    ../../PyVEnvs/Py311/lib/python3.11/site-packages/verbecc/inflector.py:259: KeyError

    Error was occuring because the "<Subvuntivo>" was empty in the "abol:ir" template.
    """
    result = cg.conjugate("abolir")
    assert result is not None


def test_inflector_es_get_conj_obs():
    co = cg._get_conj_obs("abañar")
    assert co.verb.infinitive == "abañar"
    assert co.verb_stem == "abañ"


def test_inflector_es_get_verb_stem_from_template_name():
    verb_stem = cg._inflector._get_verb_stem_from_template_name("abañar", "cort:ar")
    assert verb_stem == "abañ"


def test_inflector_es_conjugate_simple_mood_tense():
    mood = "indicativo"
    tense = "présent"
    verb_stem = "abañ"
    tense_elem = etree.fromstring(
        """<presente>
            <p><i>o</i></p>
            <p><i>as</i></p>
            <p><i>a</i></p>
            <p><i>amos</i></p>
            <p><i>áis</i></p>
            <p><i>an</i></p>
        </presente>""",
        parser=None,
    )
    tense_template = TenseTemplate(Lang.es, mood, tense_elem)
    out = cg._conjugate_simple_mood_tense(verb_stem, mood, tense, tense_template)
    assert len(out) == 6
    assert out == [
        "yo abaño",
        "tú abañas",
        "él abaña",
        "nosotros abañamos",
        "vosotros abañáis",
        "ellos abañan",
    ]


@pytest.mark.parametrize(
    "person,gender,is_reflexive,expected_result",
    [
        (Person.FirstPersonSingular, Gender.m, False, "yo"),
        (Person.FirstPersonSingular, Gender.m, True, "yo me"),
        (Person.SecondPersonSingular, Gender.m, False, "tú"),
        (Person.SecondPersonSingular, Gender.m, True, "tú te"),
        (Person.ThirdPersonSingular, Gender.m, False, "él"),
        (Person.ThirdPersonSingular, Gender.m, True, "él se"),
        (Person.ThirdPersonSingular, Gender.f, False, "ella"),
        (Person.ThirdPersonSingular, Gender.f, True, "ella se"),
        (Person.FirstPersonPlural, Gender.m, False, "nosotros"),
        (Person.FirstPersonPlural, Gender.m, True, "nosotros nos"),
        (Person.SecondPersonPlural, Gender.m, False, "vosotros"),
        (Person.SecondPersonPlural, Gender.m, True, "vosotros os"),
        (Person.ThirdPersonPlural, Gender.m, False, "ellos"),
        (Person.ThirdPersonPlural, Gender.m, True, "ellos se"),
        (Person.ThirdPersonPlural, Gender.f, False, "ellas"),
        (Person.ThirdPersonPlural, Gender.f, True, "ellas se"),
    ],
)
def test_inflector_es_get_default_pronoun(
    person: Person, gender: Gender, is_reflexive: bool, expected_result: str
):
    assert (
        cg._inflector.get_default_pronoun(person, gender, is_reflexive=is_reflexive)
        == expected_result
    )


def test_inflector_es_conjugate_mood_indicativo_tense_presente_ar_no_voseo():
    assert cg.conjugate_mood_tense("hablar", Mood.es.Indicativo, Tense.es.Presente) == [
        "yo hablo",
        "tú hablas",
        "él habla",
        "nosotros hablamos",
        "vosotros habláis",
        "ellos hablan",
    ]


def test_inflector_es_conjugate_mood_indicativo_tense_presente_ar_voseo_tipo_3():
    assert cg.conjugate_mood_tense(
        "hablar",
        Mood.es.Indicativo,
        Tense.es.Presente,
        lang_specific_options=LangSpecificOptionsEs(
            voseo_options=VoseoOptions.VoseoTipo3
        ),
    ) == [
        "yo hablo",
        "vos hablás",
        "él habla",
        "nosotros hablamos",
        "vosotros habláis",
        "ellos hablan",
    ]


def test_inflector_es_conjugate_mood_indicativo_tense_presente_er_no_voseo():
    assert cg.conjugate_mood_tense("beber", Mood.es.Indicativo, Tense.es.Presente) == [
        "yo bebo",
        "tú bebes",
        "él bebe",
        "nosotros bebemos",
        "vosotros bebéis",
        "ellos beben",
    ]


def test_inflector_es_conjugate_mood_indicativo_tense_presente_er_voseo_tipo_3():
    assert cg.conjugate_mood_tense(
        "beber",
        Mood.es.Indicativo,
        Tense.es.Presente,
        lang_specific_options=LangSpecificOptionsEs(
            voseo_options=VoseoOptions.VoseoTipo3
        ),
    ) == [
        "yo bebo",
        "vos bebés",
        "él bebe",
        "nosotros bebemos",
        "vosotros bebéis",
        "ellos beben",
    ]


def test_inflector_es_conjugate_mood_indicativo_tense_presente_ir_no_voseo():
    assert cg.conjugate_mood_tense("dormir", Mood.es.Indicativo, Tense.es.Presente) == [
        "yo duermo",
        "tú duermes",
        "él duerme",
        "nosotros dormimos",
        "vosotros dormís",
        "ellos duermen",
    ]


def test_inflector_es_conjugate_mood_indicativo_tense_presente_ir_voseo_tipo_3():
    assert cg.conjugate_mood_tense(
        "dormir",
        Mood.es.Indicativo,
        Tense.es.Presente,
        lang_specific_options=LangSpecificOptionsEs(
            voseo_options=VoseoOptions.VoseoTipo3
        ),
    ) == [
        "yo duermo",
        "vos dormís",
        "él duerme",
        "nosotros dormimos",
        "vosotros dormís",
        "ellos duermen",
    ]


def test_inflector_es_conjugate_mood_imperativo_tense_afirmativo_ar_no_voseo():
    assert cg.conjugate_mood_tense(
        "hablar", Mood.es.Imperativo, Tense.es.Afirmativo
    ) == [
        "habla",
        "hable",
        "hablemos",
        "hablad",
        "hablen",
    ]


def test_inflector_es_conjugate_mood_imperativo_tense_negativo_ar_no_voseo():
    assert cg.conjugate_mood_tense("hablar", Mood.es.Imperativo, Tense.es.Negativo) == [
        "no hables",
        "no hable",
        "no hablemos",
        "no habléis",
        "no hablen",
    ]


def test_inflector_es_conjugate_mood_imperativo_tense_afirmativo_ar_voseo_tipo_3():
    assert cg.conjugate_mood_tense(
        "hablar",
        Mood.es.Imperativo,
        Tense.es.Afirmativo,
        lang_specific_options=LangSpecificOptionsEs(
            voseo_options=VoseoOptions.VoseoTipo3
        ),
    ) == [
        "hablá",
        "hable",
        "hablemos",
        "hablad",
        "hablen",
    ]


def test_inflector_es_conjugate_mood_imperativo_tense_negativo_ar_voseo_tipo_3():
    assert cg.conjugate_mood_tense(
        "hablar",
        Mood.es.Imperativo,
        Tense.es.Negativo,
        lang_specific_options=LangSpecificOptionsEs(
            voseo_options=VoseoOptions.VoseoTipo3
        ),
    ) == [
        "no hables",
        "no hable",
        "no hablemos",
        "no habléis",
        "no hablen",
    ]


def test_inflector_es_conjugate_mood_imperativo_tense_afirmativo_ir_no_voseo():
    assert cg.conjugate_mood_tense(
        "vivir", Mood.es.Imperativo, Tense.es.Afirmativo
    ) == [
        "vive",
        "viva",
        "vivamos",
        "vivid",
        "vivan",
    ]


def test_inflector_es_conjugate_mood_imperativo_tense_negativo_ir_no_voseo():
    assert cg.conjugate_mood_tense("vivir", Mood.es.Imperativo, Tense.es.Negativo) == [
        "no vivas",
        "no viva",
        "no vivamos",
        "no viváis",
        "no vivan",
    ]


def test_inflector_es_conjugate_mood_imperativo_tense_afirmativo_ir_voseo_tipo_3():
    assert cg.conjugate_mood_tense(
        "vivir",
        Mood.es.Imperativo,
        Tense.es.Afirmativo,
        lang_specific_options=LangSpecificOptionsEs(
            voseo_options=VoseoOptions.VoseoTipo3
        ),
    ) == [
        "viví",
        "viva",
        "vivamos",
        "vivid",
        "vivan",
    ]


def test_inflector_es_conjugate_mood_imperativo_tense_negativo_ir_voseo_tipo_3():
    assert cg.conjugate_mood_tense(
        "vivir",
        Mood.es.Imperativo,
        Tense.es.Negativo,
        lang_specific_options=LangSpecificOptionsEs(
            voseo_options=VoseoOptions.VoseoTipo3
        ),
    ) == [
        "no vivas",
        "no viva",
        "no vivamos",
        "no viváis",
        "no vivan",
    ]


def test_inflector_es_conjugate_mood_imperativo_tense_afirmativo_er_no_voseo():
    assert cg.conjugate_mood_tense(
        "beber", Mood.es.Imperativo, Tense.es.Afirmativo
    ) == [
        "bebe",
        "beba",
        "bebamos",
        "bebed",
        "beban",
    ]


def test_inflector_es_conjugate_mood_imperativo_tense_negativo_er_no_voseo():
    assert cg.conjugate_mood_tense("beber", Mood.es.Imperativo, Tense.es.Negativo) == [
        "no bebas",
        "no beba",
        "no bebamos",
        "no bebáis",
        "no beban",
    ]


def test_inflector_es_conjugate_mood_imperativo_tense_afirmativo_er_voseo_tipo_3():
    assert cg.conjugate_mood_tense(
        "beber",
        Mood.es.Imperativo,
        Tense.es.Afirmativo,
        lang_specific_options=LangSpecificOptionsEs(
            voseo_options=VoseoOptions.VoseoTipo3
        ),
    ) == [
        "bebé",
        "beba",
        "bebamos",
        "bebed",
        "beban",
    ]


def test_inflector_es_conjugate_mood_imperativo_tense_negativo_er_voseo_tipo_3():
    assert cg.conjugate_mood_tense(
        "beber",
        Mood.es.Imperativo,
        Tense.es.Negativo,
        lang_specific_options=LangSpecificOptionsEs(
            voseo_options=VoseoOptions.VoseoTipo3
        ),
    ) == [
        "no bebas",
        "no beba",
        "no bebamos",
        "no bebáis",
        "no beban",
    ]


def test_inflector_es_conjugate_mood_imperativo_tense_afirmativo_ser_no_voseo():
    assert cg.conjugate_mood_tense("ser", Mood.es.Imperativo, Tense.es.Afirmativo) == [
        "sé",
        "sea",
        "seamos",
        "sed",
        "sean",
    ]


def test_inflector_es_conjugate_mood_imperativo_tense_negativo_ser_no_voseo():
    assert cg.conjugate_mood_tense("ser", Mood.es.Imperativo, Tense.es.Negativo) == [
        "no seas",
        "no sea",
        "no seamos",
        "no seáis",
        "no sean",
    ]


def test_inflector_es_conjugate_mood_imperativo_tense_afirmativo_ser_voseo_tipo_3():
    assert cg.conjugate_mood_tense(
        "ser",
        Mood.es.Imperativo,
        Tense.es.Afirmativo,
        lang_specific_options=LangSpecificOptionsEs(
            voseo_options=VoseoOptions.VoseoTipo3
        ),
    ) == [
        "sé",
        "sea",
        "seamos",
        "sed",
        "sean",
    ]


def test_inflector_es_conjugate_mood_imperativo_tense_negativo_ser_voseo_tipo_3():
    assert cg.conjugate_mood_tense(
        "ser",
        Mood.es.Imperativo,
        Tense.es.Negativo,
        lang_specific_options=LangSpecificOptionsEs(
            voseo_options=VoseoOptions.VoseoTipo3
        ),
    ) == [
        "no seas",
        "no sea",
        "no seamos",
        "no seáis",
        "no sean",
    ]



import pytest
from typing import List

from verbecc.src.defs.types.gender import Gender
from verbecc.src.defs.types.person import Person
from verbecc.src.defs.types.mood import Mood
from verbecc.src.defs.types.tense import Tense
from verbecc.src.conjugator.conjugator import Conjugator


@pytest.fixture(scope="module")
def cg():
    cg = Conjugator(lang="pt")
    yield cg


def test_all_verbs_have_templates(cg: Conjugator):
    verbs = cg.get_verbs()
    template_names = cg.get_template_names()
    missing_templates = set()
    for verb in verbs:
        if verb.template not in template_names:
            missing_templates.add(verb.template)
    assert len(missing_templates) == 0


@pytest.mark.parametrize(
    "infinitive,mood,tense,expected_result",
    [
        (
            "ter",
            "indicativo",
            "presente",
            ["eu tenho", "tu tens", "ele tem", "nós temos", "vós tendes", "eles têm"],
        ),
        (
            "ter",
            "indicativo",
            "pretérito-perfeito",
            [
                "eu tive",
                "tu tiveste",
                "ele teve",
                "nós tivemos",
                "vós tivestes",
                "eles tiveram",
            ],
        ),
        (
            "ter",
            "indicativo",
            "pretérito-imperfeito",
            [
                "eu tinha",
                "tu tinhas",
                "ele tinha",
                "nós tínhamos",
                "vós tínheis",
                "eles tinham",
            ],
        ),
        (
            "ter",
            "indicativo",
            "pretérito-mais-que-perfeito",
            [
                "eu tivera",
                "tu tiveras",
                "ele tivera",
                "nós tivéramos",
                "vós tivéreis",
                "eles tiveram",
            ],
        ),
        (
            "ter",
            "indicativo",
            "pretérito-perfeito-composto",
            [
                "eu tenho tido",
                "tu tens tido",
                "ele tem tido",
                "nós temos tido",
                "vós tendes tido",
                "eles têm tido",
            ],
        ),
        (
            "ter",
            "indicativo",
            "pretérito-mais-que-perfeito-composto",
            [
                "eu tinha tido",
                "tu tinhas tido",
                "ele tinha tido",
                "nós tínhamos tido",
                "vós tínheis tido",
                "eles tinham tido",
            ],
        ),
        (
            "ter",
            "indicativo",
            "pretérito-mais-que-perfeito-anterior",
            [
                "eu tivera tido",
                "tu tiveras tido",
                "ele tivera tido",
                "nós tivéramos tido",
                "vós tivéreis tido",
                "eles tiveram tido",
            ],
        ),
        (
            "ter",
            "indicativo",
            "futuro-do-presente",
            [
                "eu terei",
                "tu terás",
                "ele terá",
                "nós teremos",
                "vós tereis",
                "eles terão",
            ],
        ),
        (
            "ter",
            "indicativo",
            "futuro-do-presente-composto",
            [
                "eu terei tido",
                "tu terás tido",
                "ele terá tido",
                "nós teremos tido",
                "vós tereis tido",
                "eles terão tido",
            ],
        ),
        (
            "ter",
            "subjuntivo",
            "presente",
            [
                "que eu tenha",
                "que tu tenhas",
                "que ele tenha",
                "que nós tenhamos",
                "que vós tenhais",
                "que eles tenham",
            ],
        ),
        (
            "ter",
            "subjuntivo",
            "pretérito-perfeito",
            [
                "eu tenha tido",
                "tu tenhas tido",
                "ele tenha tido",
                "nós tenhamos tido",
                "vós tenhais tido",
                "eles tenham tido",
            ],
        ),
        (
            "ter",
            "subjuntivo",
            "pretérito-imperfeito",
            [
                "se eu tivesse",
                "se tu tivesses",
                "se ele tivesse",
                "se nós tivéssemos",
                "se vós tivésseis",
                "se eles tivessem",
            ],
        ),
        (
            "ter",
            "subjuntivo",
            "pretérito-mais-que-perfeito",
            [
                "eu tivesse tido",
                "tu tivesses tido",
                "ele tivesse tido",
                "nós tivéssemos tido",
                "vós tivésseis tido",
                "eles tivessem tido",
            ],
        ),
        (
            "ter",
            "subjuntivo",
            "futuro",
            [
                "quando eu tiver",
                "quando tu tiveres",
                "quando ele tiver",
                "quando nós tivermos",
                "quando vós tiverdes",
                "quando eles tiverem",
            ],
        ),
        (
            "ter",
            "subjuntivo",
            "futuro-composto",
            [
                "eu tiver tido",
                "tu tiveres tido",
                "ele tiver tido",
                "nós tivermos tido",
                "vós tiverdes tido",
                "eles tiverem tido",
            ],
        ),
        (
            "ter",
            "condicional",
            "futuro-do-pretérito",
            [
                "eu teria",
                "tu terias",
                "ele teria",
                "nós teríamos",
                "vós teríeis",
                "eles teriam",
            ],
        ),
        (
            "ter",
            "condicional",
            "futuro-do-pretérito-composto",
            [
                "eu teria tido",
                "tu terias tido",
                "ele teria tido",
                "nós teríamos tido",
                "vós teríeis tido",
                "eles teriam tido",
            ],
        ),
        (
            "ter",
            "infinitivo",
            "infinitivo-pessoal-presente",
            [
                "por ter eu",
                "por teres tu",
                "por ter ele",
                "por termos nós",
                "por terdes vós",
                "por terem eles",
            ],
        ),
        (
            "ter",
            "infinitivo",
            "infinitivo-pessoal-composto",
            [
                "ter tido",
                "teres tido",
                "ter tido",
                "termos tido",
                "terdes tido",
                "terem tido",
            ],
        ),
        (
            "ter",
            "imperativo",
            "afirmativo",
            ["-", "tem tu", "tenha você", "tenhamos nós", "tende vós", "tenham vocês"],
        ),
        (
            "ter",
            "imperativo",
            "negativo",
            [
                "-",
                "não tenhas tu",
                "não tenha você",
                "não tenhamos nós",
                "não tenhais vós",
                "não tenham vocês",
            ],
        ),
        (
            "andar",
            "indicativo",
            "pretérito-perfeito",
            [
                "eu andei",
                "tu andaste",
                "ele andou",
                "nós andámos",
                "vós andastes",
                "eles andaram",
            ],
        ),
        (
            "ficar",
            "indicativo",
            "pretérito-perfeito",
            [
                "eu fiquei",
                "tu ficaste",
                "ele ficou",
                "nós ficámos",
                "vós ficastes",
                "eles ficaram",
            ],
        ),
        (
            "amar",
            "indicativo",
            "pretérito-perfeito",
            [
                "eu amei",
                "tu amaste",
                "ele amou",
                "nós amámos",
                "vós amastes",
                "eles amaram",
            ],
        ),
        (
            "odiar",
            "indicativo",
            "pretérito-perfeito",
            [
                "eu odiei",
                "tu odiaste",
                "ele odiou",
                "nós odiámos",
                "vós odiastes",
                "eles odiaram",
            ],
        ),
        (
            "arguir",
            "indicativo",
            "presente",
            [
                "eu arguo",
                "tu argúis",
                "ele argúi",
                "nós arguimos",
                "vós arguistes",
                "eles argúem",
            ],
        ),
        (
            "arguir",
            "indicativo",
            "pretérito-perfeito",
            [
                "eu argui",
                "tu arguiste",
                "ele arguiu",
                "nós arguimos",
                "vós arguistes",
                "eles arguiram",
            ],
        ),
        (
            "arguir",
            "indicativo",
            "pretérito-imperfeito",
            [
                "eu arguia",
                "tu arguas",
                "ele arguia",
                "nós arguíamos",
                "vós arguíeis",
                "eles arguiam",
            ],
        ),
        (
            "arguir",
            "indicativo",
            "pretérito-mais-que-perfeito",
            [
                "eu arguira",
                "tu arguiras",
                "ele arguira",
                "nós arguíramos",
                "vós arguíreis",
                "eles arguiram",
            ],
        ),
        (
            "arguir",
            "indicativo",
            "futuro-do-presente",
            [
                "eu arguirei",
                "tu arguirás",
                "ele arguirá",
                "nós arguiremos",
                "vós arguireis",
                "eles arguirão",
            ],
        ),
        (
            "arguir",
            "condicional",
            "futuro-do-pretérito",
            [
                "eu arguiria",
                "tu arguirias",
                "ele arguiria",
                "nós arguiríamos",
                "vós arguiríeis",
                "eles arguiriam",
            ],
        ),
        (
            "arguir",
            "subjuntivo",
            "pretérito-imperfeito",
            [
                "se eu arguisse",
                "se tu arguisses",
                "se ele arguisse",
                "se nós arguíssemos",
                "se vós arguísseis",
                "se eles arguissem",
            ],
        ),
        (
            "arguir",
            "subjuntivo",
            "futuro",
            [
                "quando eu arguir",
                "quando tu arguires",
                "quando ele arguir",
                "quando nós arguirmos",
                "quando vós arguirdes",
                "quando eles arguirem",
            ],
        ),
        (
            "arguir",
            "infinitivo",
            "infinitivo-pessoal-presente",
            [
                "por arguir eu",
                "por arguíres tu",
                "por arguir ele",
                "por arguirmos nós",
                "por arguirdes vós",
                "por arguírem eles",
            ],
        ),
        (
            "arguir",
            "imperativo",
            "afirmativo",
            [
                "-",
                "argúi tu",
                "argua você",
                "arguamos nós",
                "arguí vós",
                "arguam vocês",
            ],
        ),
    ],
)
def test_inflector_pt_conjugate_mood_tense(
    cg: Conjugator,
    infinitive: str,
    mood: Mood,
    tense: Tense,
    expected_result: List[str],
):
    assert cg.conjugate_mood_tense(infinitive, mood, tense) == expected_result


@pytest.mark.parametrize(
    "person,gender,is_reflexive,expected_result",
    [
        (Person.FirstPersonSingular, Gender.m, False, "eu"),
        (Person.FirstPersonSingular, Gender.m, True, "eu me"),
        (Person.SecondPersonSingular, Gender.m, False, "tu"),
        (Person.SecondPersonSingular, Gender.m, True, "tu te"),
        (Person.ThirdPersonSingular, Gender.m, False, "ele"),
        (Person.ThirdPersonSingular, Gender.m, True, "ele se"),
        (Person.ThirdPersonSingular, Gender.f, False, "ela"),
        (Person.ThirdPersonSingular, Gender.f, True, "ela se"),
        (Person.FirstPersonPlural, Gender.m, False, "nós"),
        (Person.FirstPersonPlural, Gender.m, True, "nós nos"),
        (Person.SecondPersonPlural, Gender.m, False, "vós"),
        (Person.SecondPersonPlural, Gender.m, True, "vós se"),
        (Person.ThirdPersonPlural, Gender.m, False, "eles"),
        (Person.ThirdPersonPlural, Gender.m, True, "eles se"),
        (Person.ThirdPersonPlural, Gender.f, False, "elas"),
        (Person.ThirdPersonPlural, Gender.f, True, "elas se"),
    ],
)
def test_inflector_pt_get_default_pronoun(
    cg, person: Person, gender: Gender, is_reflexive: bool, expected_result: str
):
    assert (
        cg._inflector.get_default_pronoun(person, gender, is_reflexive=is_reflexive)
        == expected_result
    )



from __future__ import print_function

from bisect import bisect_left

try:
    from lxml import etree
except ImportError:
    import xml.etree.ElementTree as etree
from importlib_resources import as_file, files

# import gzip
import os

# import tempfile
from typing import List

from verbecc.src.parsers.conjugation_template import ConjugationTemplate
from verbecc.src.defs.types.lang_code import LangCodeISO639_1
from verbecc.src.defs.types.exceptions import (
    ConjugationsParserError,
    TemplateNotFoundError,
)


class ConjugationsParser:
    def __init__(self, lang: LangCodeISO639_1 = LangCodeISO639_1.fr) -> None:
        self.templates: List[ConjugationTemplate] = []
        parser = etree.XMLParser(
            dtd_validation=True, encoding="utf-8", remove_blank_text=True, remove_comments=True  # type: ignore
        )
        source = files("verbecc.data.xml.conjugations").joinpath(
            f"conjugations-{lang}.xml"
        )
        with as_file(source) as fp:
            """
            with gzip.open(fp, "rt") as zf:
                with tempfile.NamedTemporaryFile(
                    prefix=f"/tmp/conjugations-{lang}.xml.out.",
                    suffix=".xml",
                    mode="wt+",
                    encoding="utf-8",
                    delete=True,
                ) as tf:
                    next(zf)  # Skips the first line (gzip header plus xml header)
                    # Regenerate xml header
                    tf.write('<?xml version="1.0" encoding="utf-8"?>' + os.linesep)
                    for line in zf:
                        # there are some null bytes at the end that must be stripped
                        for byte in line:
                            if not byte.endswith("\x00"):
                                tf.write(byte)
                    tf.flush()
                    tree = etree.parse(
                        tf.name,
                        parser,  # type: ignore
                    )
            """
            tree = etree.parse(fp, parser)  # type: ignore
            root = tree.getroot()
            root_tag = "conjugation-{}".format(lang)
            if root.tag != root_tag:
                raise ConjugationsParserError(
                    "Root XML Tag {} Not Found".format(root_tag)
                )
            for child in root:
                if child.tag == "template":
                    self.templates.append(
                        ConjugationTemplate(lang=lang, template_elem=child)
                    )
            self.templates = sorted(self.templates, key=lambda x: x.name)
            self._keys = [template.name for template in self.templates]

    def find_template(self, name: str) -> ConjugationTemplate:
        """Assumes templates are already sorted by name"""
        i = bisect_left(self._keys, name)
        if i != len(self._keys) and self._keys[i] == name:
            return self.templates[i]
        raise TemplateNotFoundError


if __name__ == "__main__":
    conj = ConjugationsParser()



import sys

if sys.version_info >= (3, 11):
    from enum import StrEnum
else:
    from backports.strenum import StrEnum


class TenseEn(StrEnum):
    Future = "future"
    FuturePerfect = "future-perfect"
    Gerund = "gerund"
    ImperativePresent = "imperative-present"
    Imperfect = "imperfect"
    Indicative = "indicative"
    InfinitivePresent = "infinitive-present"
    PastParticiple = "past-participle"
    Pluperfect = "pluperfect"
    Present = "present"
    Preterite = "preterite"
    PreteriteAnterior = "preterite-anterior"
    PreteritePerfectCompound = "preterite-perfect-compound"
    PreteritePluperfect = "preterite-pluperfect"
    SimplePast = "simple-past"


class TenseCa(StrEnum):
    Futur = "futur"
    FuturPerfet = "futur-perfet"
    Gerundi = "gerundi"
    ImperatiuPresent = "imperatiu-present"
    Imperfet = "imperfect"
    Indicatiu = "indicatiu"
    InfinitiuPresent = "infinitiu-present"
    Particip = "particip"
    PassatSimple = "passat-simple"
    Pluscuamperfet = "pluscuamperfet"
    Present = "present"
    Pretèrit = "pretèrit"
    PretèritAnterior = "pretèrit-anterior"
    PretèritPerfetCompuest = "pretèrit-perfet-compuest"
    PretèritPluscuamperfet = "pretèrit-pluscuamperfet"


class TenseEs(StrEnum):
    Afirmativo = "afirmativo"
    Futuro = "futuro"
    FuturoPerfecto = "futuro-perfecto"
    Gerundio = "gerundio"
    Indicativo = "indicativo"
    Infinitivo = "infinitivo"
    Negativo = "negativo"
    Participo = "participo"
    Perfecto = "perfecto"
    Presente = "presente"
    PretéritoAnterior = "pretérito-anterior"
    PretéritoImperfecto = "pretérito-imperfecto"
    PretéritoImperfecto1 = "pretérito-imperfecto-1"
    PretéritoImperfecto2 = "pretérito-imperfecto-2"
    PretéritoPerfecto = "pretérito-perfecto"
    PretéritoPerfectoCompuesto = "pretérito-perfecto-compuesto"
    PretéritoPerfectoSimple = "pretérito-perfecto-simple"
    PretéritoPluscuamperfecto = "pretérito-pluscuamperfecto"
    PretéritoPluscuamperfecto1 = "pretérito-pluscuamperfecto-1"
    PretéritoPluscuamperfecto2 = "pretérito-pluscuamperfecto-2"


class TenseFr(StrEnum):
    FutureAntériuer = "futur-antérieur"
    FuturSimple = "futur-simple"
    Imparfait = "imparfait"
    ImperatifPassé = "imperatif-passé"
    ImperatifPrésent = "imperatif-présent"
    InfinitifPrésent = "infinitif-présent"
    ParticipePassé = "participe-passé"
    ParticipePresent = "participe-présent"
    Passé = "passé"
    PasséAntérieur = "passé-antérieur"
    PasséCompose = "passé-composé"
    PasséSimple = "passé-simple"
    PlusQueParfait = "plus-que-parfait"
    Présent = "présent"


class TenseIt(StrEnum):
    Affermativo = "affermativo"
    Futuro = "futuro"
    FuturoAnteriore = "futuro-anteriore"
    Gerundio = "gerundio"
    Imperfetto = "imperfetto"
    Infinitivo = "infinitivo"
    negativo = "negativo"
    Negativo = "Negativo"
    ParticipioPassato = "participio-passato"
    ParticipioPresente = "participio-presente"
    Passato = "passato"
    PassatoProssimo = "passato-prossimo"
    PassatoRemoto = "passato-remoto"
    Presente = "presente"
    Trapassato = "trapassato"
    TrapassatoProssimo = "trapassato-prossimo"
    TrapassatoRemoto = "trapassato-remoto"


class TensePt(StrEnum):
    Afirmativo = "afirmativo"
    Futuro = "futuro"
    FuturoComposto = "futuro-composto"
    FuturoDoPresente = "futuro-do-presente"
    FuturoDoPresenteComposto = "futuro-do-presente-composto"
    FuturoDoPretérito = "futuro-do-pretérito"
    FuturoDoPretéritoComposto = "futuro-do-pretérito-composto"
    FuturoPerfeito = "futuro-perfeito"
    Gerúndio = "gerúndio"
    ImperativoPresente = "imperativo-presente"
    Imperfeito = "imperfeito"
    Infinitivo = "infinitivo"
    InfinitivoPessoalComposto = "infinitivo-pessoal-composto"
    InfinitivoPessoalPresente = "infinitivo-pessoal-presente"
    InfinitivoPresente = "infinitivo-presente"
    Negativo = "negativo"
    Particípio = "particípio"
    ParticípioPassado = "particípio-passado"
    Presente = "presente"
    PretéritoImperfeito = "pretérito-imperfeito"
    PretéritoMaisQuePerfeito = "pretérito-mais-que-perfeito"
    PretéritoMaisQuePerfeitoAnterior = "pretérito-mais-que-perfeito-anterior"
    PretéritoMaisQuePerfeitoComposto = "pretérito-mais-que-perfeito-composto"
    PretéritoPerfeito = "pretérito-perfeito"
    PretéritoPerfeitoComposto = "pretérito-perfeito-composto"


class TenseRo(StrEnum):
    Afirmativ = "afirmativ"
    Indicativ = "indicativ"
    Gerunziu = "gerunziu"
    Imperativ = "imperativ"
    Imperfect = "imperfect"
    Infinitiv = "infinitiv"
    MaiMultCaPerfect = "mai-mult-ca-perfect"
    Negativ = "negativ"
    Participiu = "participiu"
    Perfect = "perfect"
    PerfectCompus = "perfect-compus"
    PerfectSimplu = "perfect-simplu"
    Prezent = "prezent"
    Viitor = "viitor"
    Viitor1 = "viitor-1"
    Viitor1Popular = "viitor-1-popular"
    Viitor2 = "viitor-2"
    Viitor2Popular = "viitor-2-popular"
    ViitorAnterior = "viitor-anterior"


class Tense:
    fr = TenseFr
    es = TenseEs
    en = TenseEn
    it = TenseIt
    ca = TenseCa
    ro = TenseRo
    pt = TensePt
